/*
 * Copyright (c) 2021.  ʕʘ̅͜ʘ̅ʔ
 * Use of this source code is governed by a
 * MIT license that can be found in the LICENSE.
 */

import 'dart:io' show Platform;

import 'package:logging/logging.dart';

class LoggingService {
  Logger? _logger;
  static int _sequenceNumber = 0;

  LoggingService([String? name]) {
    hierarchicalLoggingEnabled = true;
    _logger = name == null ? Logger.root : Logger(name);
    Logger.root.level = _getRootLevelFromEnv();
    _logger!.onRecord.listen((record) {
      print('${record.loggerName.isEmpty ? '' : '[${record.loggerName}]'}'
          '${record.level.name}: '
          '${record.sequenceNumber}-'
          '${record.time}: '
          '${record.message} '
          '${record.error == null ? '' : '\n${record.error as String}'} '
          '${record.stackTrace == null ? '' : '\n${record.stackTrace}'}');
    });
  }

  Logger? get logger {
    LoggingService._sequenceNumber++;
    return _logger;
  }

  set level(Level level) {
    _logger!.level = level;
  }

  set levelToRoot(Level level) {
    Logger.root.level = level;
  }

  Level _getRootLevelFromEnv() {
    const envName = 'PASSWD_GEN_LOG_LEVEL';
    var level = Level.WARNING;
    final envVars = Platform.environment;
    if (envVars.containsKey(envName)) {
      switch (envVars[envName]!.toUpperCase()) {
        case 'SHOUT':
          level = Level.SHOUT;
          break;
        case 'SEVERE':
          level = Level.SEVERE;
          break;
        case 'WARNING':
          level = Level.WARNING;
          break;
        case 'INFO':
          level = Level.INFO;
          break;
        case 'CONFIG':
          level = Level.CONFIG;
          break;
        case 'FINE':
          level = Level.FINE;
          break;
        case 'FINER':
          level = Level.FINER;
          break;
        case 'FINEST':
          level = Level.FINEST;
          break;
        case 'ALL':
          level = Level.ALL;
          break;
        default:
          level = Level.OFF;
      }
    }
    return level;
  }
}
